@@ -3,8 +3,8 @@
 #' Match one scene to another based on linear regression of pseudo-invariant features (PIF).
 #' 
 #' @param img RasterStack or RasterBrick. Image to be adjusted.
-#' @param ref RasterStack or RasterBruck. Reference image.
-#' @param method Method to calculate pixel similariry. Options: euclidean distance ('ed'), spectral angle ('sam') or pearson correlation coefficient ('cor').
+#' @param ref RasterStack or RasterBrick. Reference image.
+#' @param method Method to calculate pixel similarity. Options: euclidean distance ('ed'), spectral angle ('sam') or pearson correlation coefficient ('cor').
 #' @param quantile Numeric. Threshold quantile used to identify PIFs
 #' @param returnPifMap Logical. Return a binary raster map ot pixels which were identified as pesudo-invariant features.
 #' @param returnSimMap Logical. Return the similarity map as well
@@ -59,7 +59,11 @@
 #' summary(lsat_b_adj$models[[1]])
 #' }
 pifMatch <- function(img, ref, method = "cor", quantile = 0.95, returnPifMap = TRUE, returnSimMap = TRUE, returnModels = FALSE){
-    if(nlayers(img)!=nlayers(ref) | nlayers(img) <= 1) stop("Both images need at least two corresponding bands and must have the same number of bands.", call.=FALSE)
+    
+	img <- .toRaster(img)
+	ref <- .toRaster(ref)
+	
+	if(nlayers(img)!=nlayers(ref) | nlayers(img) <= 1) stop("Both images need at least two corresponding bands and must have the same number of bands.", call.=FALSE)
     
     imgfull <- img
     ## Get joint extent
