@@ -4,8 +4,8 @@
 #include <R_ext/Rdynload.h>
 
 /* FIXME: 
-   Check these declarations against the C/Fortran source code.
-*/
+ Check these declarations against the C/Fortran source code.
+ */
 
 /* .Call calls */
 extern SEXP _RStoolbox_classQA(SEXP, SEXP);
@@ -15,7 +15,7 @@ extern SEXP _RStoolbox_nnls_solver(SEXP, SEXP, SEXP, SEXP);
 extern SEXP _RStoolbox_normImageCpp(SEXP, SEXP, SEXP);
 extern SEXP _RStoolbox_oneHotCpp(SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP _RStoolbox_predictMlcCpp(SEXP, SEXP, SEXP);
-extern SEXP _RStoolbox_predKmeansCpp(SEXP, SEXP);
+extern SEXP _RStoolbox_predKmeansCpp(SEXP, SEXP, SEXP);
 extern SEXP _RStoolbox_pwSimilarityCpp(SEXP, SEXP, SEXP);
 extern SEXP _RStoolbox_rescaleImageCpp(SEXP, SEXP, SEXP, SEXP);
 extern SEXP _RStoolbox_specSimC(SEXP, SEXP);
@@ -29,7 +29,7 @@ static const R_CallMethodDef CallEntries[] = {
     {"_RStoolbox_normImageCpp",       (DL_FUNC) &_RStoolbox_normImageCpp,        3},
     {"_RStoolbox_oneHotCpp",          (DL_FUNC) &_RStoolbox_oneHotCpp,           5},
     {"_RStoolbox_predictMlcCpp",      (DL_FUNC) &_RStoolbox_predictMlcCpp,       3},
-    {"_RStoolbox_predKmeansCpp",      (DL_FUNC) &_RStoolbox_predKmeansCpp,       2},
+    {"_RStoolbox_predKmeansCpp",      (DL_FUNC) &_RStoolbox_predKmeansCpp,       3},
     {"_RStoolbox_pwSimilarityCpp",    (DL_FUNC) &_RStoolbox_pwSimilarityCpp,     3},
     {"_RStoolbox_rescaleImageCpp",    (DL_FUNC) &_RStoolbox_rescaleImageCpp,     4},
     {"_RStoolbox_specSimC",           (DL_FUNC) &_RStoolbox_specSimC,            2},
