@@ -1,5 +1,4 @@
 context("readMeta & stackMeta")
-
 mfil <- list.files("testdata/metadata", full = TRUE)
 for(f in mfil) {
 	test_that(paste("readMeta and summary(readMeta)", basename(f)), {
@@ -7,6 +6,8 @@ for(f in mfil) {
 				expect_s3_class(m$ACQUISITION_DATE, c("POSIXlt", "POSIXt")) 
                 expect_true(all(grepl("^B1?[0-9]_(dn|toa|sre){1}$", m$DATA$BANDS[1:4])))
 				expect_output( summary(m), "Scene:")
+				expect_is(m <- readMeta(f, raw = TRUE), "list")
+				expect_gte(length(names(m)), 2)
 			})
 	
 }
